/// QasTools: Desktop toolset for the Linux sound system ALSA.
/// \copyright See COPYING file.

#include "cmd_options.hpp"

CMD_Options::CMD_Options ()
: start_tray_minimized ( false )
, start_single_instance ( true )
{
}

CMD_Options::~CMD_Options () = default;
