/// QasTools: Desktop toolset for the Linux sound system ALSA.
/// \copyright See COPYING file.

#include "mixer_hctl_setup.hpp"

namespace Views
{

Mixer_HCTL_Setup::Mixer_HCTL_Setup ( Utility::Setup * parent_n )
: Views::View_Base_Setup ( parent_n )
{
}

Mixer_HCTL_Setup::~Mixer_HCTL_Setup () = default;

bool
Mixer_HCTL_Setup::set_splitter_state ( const QByteArray & state_n )
{
  return change ( _splitter_state, state_n );
}

bool
Mixer_HCTL_Setup::set_iface_name ( const QString & name_n )
{
  return change ( _iface_name, name_n );
}

bool
Mixer_HCTL_Setup::set_elem_grp_name ( const QString & name_n )
{
  return change ( _elem_grp_name, name_n );
}

bool
Mixer_HCTL_Setup::set_elem_grp_index ( unsigned int index_n )
{
  return change ( _elem_grp_index, index_n );
}

} // namespace Views
