/// QasTools: Desktop toolset for the Linux sound system ALSA.
/// \copyright See COPYING file.

#include "info_texts.hpp"

const char info_text_options[] = "\
Options:\n\
  -h, --help          prints this help\n\
  -i, --copying       prints copying information\n\
  -v, --version       prints the program version\n\
";
