/// QasTools: Desktop toolset for the Linux sound system ALSA.
/// \copyright See COPYING file.

#include "device_settings_dialog_setup.hpp"

namespace Views
{

Device_Settings_Dialog_Setup::Device_Settings_Dialog_Setup (
    Utility::Setup * parent_n )
: Utility::Setup ( parent_n )
{
}

Device_Settings_Dialog_Setup::~Device_Settings_Dialog_Setup () = default;

bool
Device_Settings_Dialog_Setup::set_page ( std::size_t page_n )
{
  return change ( _page, page_n );
}

bool
Device_Settings_Dialog_Setup::set_window_geometry ( const QByteArray & geom_n )
{
  return change ( _window_geometry, geom_n );
}

bool
Device_Settings_Dialog_Setup::set_splitter_state ( const QByteArray & state_n )
{
  return change ( _splitter_state, state_n );
}

bool
Device_Settings_Dialog_Setup::set_controls_header_view_state (
    const QByteArray & state_n )
{
  return change ( _controls_header_view_state, state_n );
}

} // namespace Views
