/// QasTools: Desktop toolset for the Linux sound system ALSA.
/// \copyright See COPYING file.

#pragma once

#include "views/view_base_setup.hpp"

namespace Views
{

/// @brief Mixer_Setup
///
class Mixer_Setup : public Views::View_Base_Setup
{
  public:
  // -- Construction

  Mixer_Setup ( Utility::Setup * parent_n );

  ~Mixer_Setup () override;

  // -- Show slider value labels

  bool
  show_slider_value_labels () const
  {
    return _show_slider_value_labels;
  }

  bool
  set_show_slider_value_labels ( bool flag_n );

  // -- Show tool tips

  bool
  show_tool_tips () const
  {
    return _show_tool_tips;
  }

  bool
  set_show_tool_tips ( bool flag_n );

  private:
  // -- Attributes
  bool _show_slider_value_labels = true;
  bool _show_tool_tips = true;
};

} // namespace Views
