/// QasTools: Desktop toolset for the Linux sound system ALSA.
/// \copyright See COPYING file.

#include "device_selection_view_setup.hpp"

namespace Views
{

Device_Selection_View_Setup::Device_Selection_View_Setup (
    Utility::Setup * parent_n )
: Utility::Setup ( parent_n )
{
}

Device_Selection_View_Setup::~Device_Selection_View_Setup () = default;

bool
Device_Selection_View_Setup::set_user_device ( const QString & device_n )
{
  return change ( _user_device, device_n );
}

} // namespace Views
