/// QasTools: Desktop toolset for the Linux sound system ALSA.
/// \copyright See COPYING file.

#include "main_window_setup.hpp"

Main_Window_Setup::Main_Window_Setup ()
: Utility::Setup ( nullptr )
, mixer_dev ( this )
, inputs ( this )
, hctl ( this )
, dev_select ( this )
{
}

Main_Window_Setup::~Main_Window_Setup () = default;

bool
Main_Window_Setup::set_show_dev_select ( bool flag_n )
{
  return change ( _show_dev_select, flag_n );
}