/****************************************************************************
** Meta object code from reading C++ file 'pad.hpp'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../qastools-v1.5.0/shared/src/wdg/sliders_pad/pad.hpp"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'pad.hpp' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN3Wdg11Sliders_Pad3PadE_t {};
} // unnamed namespace

template <> constexpr inline auto Wdg::Sliders_Pad::Pad::qt_create_metaobjectdata<qt_meta_tag_ZN3Wdg11Sliders_Pad3PadE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Wdg::Sliders_Pad::Pad",
        "sig_focus_changed",
        "",
        "sig_footer_label_selected",
        "group_idx_n",
        "column_idx_n",
        "set_focus_proxy",
        "row_idx_n",
        "header_label_selected",
        "footer_label_selected",
        "style_palette_changed",
        "style_id_n"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'sig_focus_changed'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'sig_footer_label_selected'
        QtMocHelpers::SignalData<void(unsigned int, unsigned int)>(3, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::UInt, 4 }, { QMetaType::UInt, 5 },
        }}),
        // Slot 'set_focus_proxy'
        QtMocHelpers::SlotData<bool(unsigned int)>(6, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { QMetaType::UInt, 4 },
        }}),
        // Slot 'set_focus_proxy'
        QtMocHelpers::SlotData<bool(unsigned int, unsigned int, unsigned int)>(6, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { QMetaType::UInt, 4 }, { QMetaType::UInt, 5 }, { QMetaType::UInt, 7 },
        }}),
        // Slot 'header_label_selected'
        QtMocHelpers::SlotData<void(unsigned int, unsigned int)>(8, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::UInt, 4 }, { QMetaType::UInt, 5 },
        }}),
        // Slot 'footer_label_selected'
        QtMocHelpers::SlotData<void(unsigned int, unsigned int)>(9, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::UInt, 4 }, { QMetaType::UInt, 5 },
        }}),
        // Slot 'style_palette_changed'
        QtMocHelpers::SlotData<void(unsigned int)>(10, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::UInt, 11 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<Pad, qt_meta_tag_ZN3Wdg11Sliders_Pad3PadE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject Wdg::Sliders_Pad::Pad::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3Wdg11Sliders_Pad3PadE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3Wdg11Sliders_Pad3PadE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3Wdg11Sliders_Pad3PadE_t>.metaTypes,
    nullptr
} };

void Wdg::Sliders_Pad::Pad::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<Pad *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->sig_focus_changed(); break;
        case 1: _t->sig_footer_label_selected((*reinterpret_cast<std::add_pointer_t<uint>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<uint>>(_a[2]))); break;
        case 2: { bool _r = _t->set_focus_proxy((*reinterpret_cast<std::add_pointer_t<uint>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 3: { bool _r = _t->set_focus_proxy((*reinterpret_cast<std::add_pointer_t<uint>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<uint>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<uint>>(_a[3])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 4: _t->header_label_selected((*reinterpret_cast<std::add_pointer_t<uint>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<uint>>(_a[2]))); break;
        case 5: _t->footer_label_selected((*reinterpret_cast<std::add_pointer_t<uint>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<uint>>(_a[2]))); break;
        case 6: _t->style_palette_changed((*reinterpret_cast<std::add_pointer_t<uint>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (Pad::*)()>(_a, &Pad::sig_focus_changed, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (Pad::*)(unsigned int , unsigned int )>(_a, &Pad::sig_footer_label_selected, 1))
            return;
    }
}

const QMetaObject *Wdg::Sliders_Pad::Pad::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Wdg::Sliders_Pad::Pad::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3Wdg11Sliders_Pad3PadE_t>.strings))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int Wdg::Sliders_Pad::Pad::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 7)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 7;
    }
    return _id;
}

// SIGNAL 0
void Wdg::Sliders_Pad::Pad::sig_focus_changed()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void Wdg::Sliders_Pad::Pad::sig_footer_label_selected(unsigned int _t1, unsigned int _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2);
}
QT_WARNING_POP
