/****************************************************************************
** Meta object code from reading C++ file 'main_window.hpp'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../qastools-v1.5.0/qasmixer/src/main_window.hpp"
#include <QtGui/qtextcursor.h>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'main_window.hpp' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN11Main_WindowE_t {};
} // unnamed namespace

template <> constexpr inline auto Main_Window::qt_create_metaobjectdata<qt_meta_tag_ZN11Main_WindowE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Main_Window",
        "sig_show_settings",
        "",
        "sig_show_about",
        "sig_show_about_qt",
        "sig_control_changed",
        "sig_quit",
        "select_ctl_from_side_iface",
        "reload_mixer_device",
        "reload_mixer_inputs",
        "reload_mixer_view",
        "refresh_views",
        "set_fullscreen",
        "flag_n",
        "show_device_selection",
        "toggle_device_selection",
        "toggle_device_settings_dialog",
        "save_state",
        "on_splitter_moved"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'sig_show_settings'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'sig_show_about'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'sig_show_about_qt'
        QtMocHelpers::SignalData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'sig_control_changed'
        QtMocHelpers::SignalData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'sig_quit'
        QtMocHelpers::SignalData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'select_ctl_from_side_iface'
        QtMocHelpers::SlotData<void()>(7, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'reload_mixer_device'
        QtMocHelpers::SlotData<void()>(8, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'reload_mixer_inputs'
        QtMocHelpers::SlotData<void()>(9, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'reload_mixer_view'
        QtMocHelpers::SlotData<void()>(10, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'refresh_views'
        QtMocHelpers::SlotData<void()>(11, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'set_fullscreen'
        QtMocHelpers::SlotData<void(bool)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 13 },
        }}),
        // Slot 'show_device_selection'
        QtMocHelpers::SlotData<void(bool)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 13 },
        }}),
        // Slot 'toggle_device_selection'
        QtMocHelpers::SlotData<void()>(15, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'toggle_device_settings_dialog'
        QtMocHelpers::SlotData<void()>(16, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'save_state'
        QtMocHelpers::SlotData<void()>(17, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_splitter_moved'
        QtMocHelpers::SlotData<void()>(18, 2, QMC::AccessProtected, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<Main_Window, qt_meta_tag_ZN11Main_WindowE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject Main_Window::staticMetaObject = { {
    QMetaObject::SuperData::link<QMainWindow::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11Main_WindowE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11Main_WindowE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN11Main_WindowE_t>.metaTypes,
    nullptr
} };

void Main_Window::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<Main_Window *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->sig_show_settings(); break;
        case 1: _t->sig_show_about(); break;
        case 2: _t->sig_show_about_qt(); break;
        case 3: _t->sig_control_changed(); break;
        case 4: _t->sig_quit(); break;
        case 5: _t->select_ctl_from_side_iface(); break;
        case 6: _t->reload_mixer_device(); break;
        case 7: _t->reload_mixer_inputs(); break;
        case 8: _t->reload_mixer_view(); break;
        case 9: _t->refresh_views(); break;
        case 10: _t->set_fullscreen((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 11: _t->show_device_selection((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 12: _t->toggle_device_selection(); break;
        case 13: _t->toggle_device_settings_dialog(); break;
        case 14: _t->save_state(); break;
        case 15: _t->on_splitter_moved(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (Main_Window::*)()>(_a, &Main_Window::sig_show_settings, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (Main_Window::*)()>(_a, &Main_Window::sig_show_about, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (Main_Window::*)()>(_a, &Main_Window::sig_show_about_qt, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (Main_Window::*)()>(_a, &Main_Window::sig_control_changed, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (Main_Window::*)()>(_a, &Main_Window::sig_quit, 4))
            return;
    }
}

const QMetaObject *Main_Window::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Main_Window::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11Main_WindowE_t>.strings))
        return static_cast<void*>(this);
    return QMainWindow::qt_metacast(_clname);
}

int Main_Window::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QMainWindow::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 16)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 16;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 16)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 16;
    }
    return _id;
}

// SIGNAL 0
void Main_Window::sig_show_settings()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void Main_Window::sig_show_about()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void Main_Window::sig_show_about_qt()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void Main_Window::sig_control_changed()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void Main_Window::sig_quit()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}
QT_WARNING_POP
