/// QasTools: Desktop toolset for the Linux sound system ALSA.
/// \copyright See COPYING file.

#pragma once

#include "utility/setup.hpp"
#include <QKeySequence>
#include <QString>

namespace Views
{

/// @brief Device_Selection_View_Setup
///
class Device_Selection_View_Setup : public Utility::Setup
{
  public:
  // -- Construction

  Device_Selection_View_Setup ( Utility::Setup * parent_n );

  ~Device_Selection_View_Setup () override;

  // -- Attributes

  QKeySequence kseq_toggle_vis = Qt::Key_F6;

  // -- User device

  const QString &
  user_device () const
  {
    return _user_device;
  }

  bool
  set_user_device ( const QString & device_n );

  private:
  // -- Attributes
  QString _user_device;
};

} // namespace Views
