/// QasTools: Desktop toolset for the Linux sound system ALSA.
/// \copyright See COPYING file.

#include "setup.hpp"

namespace Tray
{

Setup::Setup ( Utility::Setup * parent_n )
: Utility::Setup ( parent_n )
, _user_device ( QLatin1StringView ( "hw:0" ) )
{
}

Setup::~Setup () = default;

bool
Setup::set_device_mode ( Mixer_Device dev_n )
{
  return change ( _device_mode, dev_n );
}

bool
Setup::set_user_device ( const QString & dev_n )
{
  return change ( _user_device, dev_n );
}

bool
Setup::set_balloon_enabled ( bool flag_n )
{
  return change ( _balloon_enabled, flag_n );
}

bool
Setup::set_balloon_lifetime ( unsigned int time_ms_n )
{
  return change ( _balloon_lifetime, time_ms_n );
}

bool
Setup::set_wheel_degrees ( unsigned int degrees_n )
{
  return change ( _wheel_degrees, degrees_n );
}

bool
Setup::set_show_volume_icon ( bool flag_n )
{
  return change ( _show_volume_icon, flag_n );
}

} // namespace Tray
