/// QasTools: Desktop toolset for the Linux sound system ALSA.
/// \copyright See COPYING file.

#include "setup.hpp"

namespace Utility
{

Setup::Setup ( Setup * parent_n )
: _parent ( parent_n )
{
}

Setup::~Setup () = default;

void
Setup::notify_change ()
{
  ++_change_id;
  if ( parent () != nullptr ) {
    parent ()->notify_change ();
  }
}

} // namespace Utility
