/****************************************************************************
** Meta object code from reading C++ file 'desktop_items.hpp'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../qastools-v1.5.0/qasmixer/src/desktop_items.hpp"
#include <QtGui/qtextcursor.h>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'desktop_items.hpp' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN13Desktop_ItemsE_t {};
} // unnamed namespace

template <> constexpr inline auto Desktop_Items::qt_create_metaobjectdata<qt_meta_tag_ZN13Desktop_ItemsE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Desktop_Items",
        "sig_quit",
        "",
        "select_ctl",
        "ctl_str_n",
        "main_mixer_raise",
        "main_mixer_close",
        "main_mixer_toggle_by_tray",
        "show_dialog_settings",
        "show_dialog_about",
        "show_dialog_about_qt",
        "commit_data",
        "QSessionManager&",
        "manager_n",
        "save_state",
        "save_settings",
        "shutdown",
        "quit",
        "main_mixer_reload_view",
        "reload_inputs_setup",
        "tray_mixer_update_visibility",
        "tray_mixer_update_icon",
        "tray_mixer_reload_mixer_dev",
        "tray_mixer_reload_current_mixer_dev",
        "tray_mixer_update_balloon_setup",
        "tray_mixer_update_input_setup"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'sig_quit'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'select_ctl'
        QtMocHelpers::SlotData<void(const QString &)>(3, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 4 },
        }}),
        // Slot 'main_mixer_raise'
        QtMocHelpers::SlotData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'main_mixer_close'
        QtMocHelpers::SlotData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'main_mixer_toggle_by_tray'
        QtMocHelpers::SlotData<void()>(7, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'show_dialog_settings'
        QtMocHelpers::SlotData<void()>(8, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'show_dialog_about'
        QtMocHelpers::SlotData<void()>(9, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'show_dialog_about_qt'
        QtMocHelpers::SlotData<void()>(10, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'commit_data'
        QtMocHelpers::SlotData<void(QSessionManager &)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 12, 13 },
        }}),
        // Slot 'save_state'
        QtMocHelpers::SlotData<void(QSessionManager &)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 12, 13 },
        }}),
        // Slot 'save_settings'
        QtMocHelpers::SlotData<void()>(15, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'shutdown'
        QtMocHelpers::SlotData<void()>(16, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'quit'
        QtMocHelpers::SlotData<void()>(17, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'main_mixer_reload_view'
        QtMocHelpers::SlotData<void()>(18, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'reload_inputs_setup'
        QtMocHelpers::SlotData<void()>(19, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'tray_mixer_update_visibility'
        QtMocHelpers::SlotData<void()>(20, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'tray_mixer_update_icon'
        QtMocHelpers::SlotData<void()>(21, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'tray_mixer_reload_mixer_dev'
        QtMocHelpers::SlotData<void()>(22, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'tray_mixer_reload_current_mixer_dev'
        QtMocHelpers::SlotData<void()>(23, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'tray_mixer_update_balloon_setup'
        QtMocHelpers::SlotData<void()>(24, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'tray_mixer_update_input_setup'
        QtMocHelpers::SlotData<void()>(25, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<Desktop_Items, qt_meta_tag_ZN13Desktop_ItemsE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject Desktop_Items::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13Desktop_ItemsE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13Desktop_ItemsE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN13Desktop_ItemsE_t>.metaTypes,
    nullptr
} };

void Desktop_Items::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<Desktop_Items *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->sig_quit(); break;
        case 1: _t->select_ctl((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 2: _t->main_mixer_raise(); break;
        case 3: _t->main_mixer_close(); break;
        case 4: _t->main_mixer_toggle_by_tray(); break;
        case 5: _t->show_dialog_settings(); break;
        case 6: _t->show_dialog_about(); break;
        case 7: _t->show_dialog_about_qt(); break;
        case 8: _t->commit_data((*reinterpret_cast<std::add_pointer_t<QSessionManager&>>(_a[1]))); break;
        case 9: _t->save_state((*reinterpret_cast<std::add_pointer_t<QSessionManager&>>(_a[1]))); break;
        case 10: _t->save_settings(); break;
        case 11: _t->shutdown(); break;
        case 12: _t->quit(); break;
        case 13: _t->main_mixer_reload_view(); break;
        case 14: _t->reload_inputs_setup(); break;
        case 15: _t->tray_mixer_update_visibility(); break;
        case 16: _t->tray_mixer_update_icon(); break;
        case 17: _t->tray_mixer_reload_mixer_dev(); break;
        case 18: _t->tray_mixer_reload_current_mixer_dev(); break;
        case 19: _t->tray_mixer_update_balloon_setup(); break;
        case 20: _t->tray_mixer_update_input_setup(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (Desktop_Items::*)()>(_a, &Desktop_Items::sig_quit, 0))
            return;
    }
}

const QMetaObject *Desktop_Items::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Desktop_Items::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13Desktop_ItemsE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int Desktop_Items::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 21)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 21;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 21)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 21;
    }
    return _id;
}

// SIGNAL 0
void Desktop_Items::sig_quit()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}
QT_WARNING_POP
