/// QasTools: Desktop toolset for the Linux sound system ALSA.
/// \copyright See COPYING file.

#include "desktop_items_dbus_adaptor.hpp"
#include <desktop_items.hpp>

Desktop_Items_DBus_Adaptor::Desktop_Items_DBus_Adaptor (
    Desktop_Items * desktop_items_n )
: QDBusAbstractAdaptor ( desktop_items_n )
, _desktop_items ( desktop_items_n )
{
}

Desktop_Items_DBus_Adaptor::~Desktop_Items_DBus_Adaptor () = default;

void
Desktop_Items_DBus_Adaptor::raise ()
{
  if ( _desktop_items != nullptr ) {
    _desktop_items->main_mixer_raise ();
  }
}

void
Desktop_Items_DBus_Adaptor::selectControl ( const QString & ctl_name_n )
{
  if ( _desktop_items != nullptr ) {
    _desktop_items->select_ctl ( ctl_name_n );
  }
}
