/// QasTools: Desktop toolset for the Linux sound system ALSA.
/// \copyright See COPYING file.

#include "mixer_setup.hpp"

namespace Views
{

Mixer_Setup::Mixer_Setup ( Utility::Setup * parent_n )
: Views::View_Base_Setup ( parent_n )
{
}

Mixer_Setup::~Mixer_Setup () = default;

bool
Mixer_Setup::set_show_slider_value_labels ( bool flag_n )
{
  return change ( _show_slider_value_labels, flag_n );
}

bool
Mixer_Setup::set_show_tool_tips ( bool flag_n )
{
  return change ( _show_tool_tips, flag_n );
}

} // namespace Views
