/// QasTools: Desktop toolset for the Linux sound system ALSA.
/// \copyright See COPYING file.

#pragma once

#include "utility/setup.hpp"

namespace Views
{

/// @brief View_Base general static setup variables
///
class View_Base_Setup : public Utility::Setup
{
  public:
  // -- Construction

  View_Base_Setup ( Utility::Setup * parent_n );

  ~View_Base_Setup () override;
};

} // namespace Views
