/// QasTools: Desktop toolset for the Linux sound system ALSA.
/// \copyright See COPYING file.

#include "settings_dialog_setup.hpp"

namespace Views
{

Settings_Dialog_Setup::Settings_Dialog_Setup ( Utility::Setup * parent_n )
: Utility::Setup ( parent_n )
{
}

Settings_Dialog_Setup::~Settings_Dialog_Setup () = default;

bool
Settings_Dialog_Setup::set_page ( std::size_t page_n )
{
  return change ( _page, page_n );
}

bool
Settings_Dialog_Setup::set_window_geometry ( const QByteArray & value_n )
{
  return change ( _window_geometry, value_n );
}

bool
Settings_Dialog_Setup::set_splitter_state ( const QByteArray & state_n )
{
  return change ( _splitter_state, state_n );
}

} // namespace Views
