/****************************************************************************
** Meta object code from reading C++ file 'settings_dialog.hpp'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../qastools-v1.5.0/qasmixer/src/views/settings_dialog.hpp"
#include <QtGui/qtextcursor.h>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'settings_dialog.hpp' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN5Views15Settings_DialogE_t {};
} // unnamed namespace

template <> constexpr inline auto Views::Settings_Dialog::qt_create_metaobjectdata<qt_meta_tag_ZN5Views15Settings_DialogE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Views::Settings_Dialog",
        "sig_change_mixer_view",
        "",
        "sig_change_input",
        "sig_change_tray_visibility",
        "sig_change_tray_icon",
        "sig_change_tray_mixer_dev",
        "sig_change_tray_balloon",
        "sig_change_tray_input",
        "sig_close",
        "wheel_turns_slider_changed",
        "value_n",
        "wheel_turns_input_changed",
        "tray_wheel_turns_slider_changed",
        "tray_wheel_turns_input_changed",
        "change_startup",
        "change_appearance",
        "change_input",
        "change_tray_visibility",
        "change_tray_icon",
        "change_tray_mixer_dev",
        "change_tray_balloon",
        "change_tray_input"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'sig_change_mixer_view'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'sig_change_input'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'sig_change_tray_visibility'
        QtMocHelpers::SignalData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'sig_change_tray_icon'
        QtMocHelpers::SignalData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'sig_change_tray_mixer_dev'
        QtMocHelpers::SignalData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'sig_change_tray_balloon'
        QtMocHelpers::SignalData<void()>(7, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'sig_change_tray_input'
        QtMocHelpers::SignalData<void()>(8, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'sig_close'
        QtMocHelpers::SignalData<void()>(9, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'wheel_turns_slider_changed'
        QtMocHelpers::SlotData<void(int)>(10, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Int, 11 },
        }}),
        // Slot 'wheel_turns_input_changed'
        QtMocHelpers::SlotData<void(double)>(12, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Double, 11 },
        }}),
        // Slot 'tray_wheel_turns_slider_changed'
        QtMocHelpers::SlotData<void(int)>(13, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Int, 11 },
        }}),
        // Slot 'tray_wheel_turns_input_changed'
        QtMocHelpers::SlotData<void(double)>(14, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Double, 11 },
        }}),
        // Slot 'change_startup'
        QtMocHelpers::SlotData<void()>(15, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'change_appearance'
        QtMocHelpers::SlotData<void()>(16, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'change_input'
        QtMocHelpers::SlotData<void()>(17, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'change_tray_visibility'
        QtMocHelpers::SlotData<void()>(18, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'change_tray_icon'
        QtMocHelpers::SlotData<void()>(19, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'change_tray_mixer_dev'
        QtMocHelpers::SlotData<void()>(20, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'change_tray_balloon'
        QtMocHelpers::SlotData<void()>(21, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'change_tray_input'
        QtMocHelpers::SlotData<void()>(22, 2, QMC::AccessProtected, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<Settings_Dialog, qt_meta_tag_ZN5Views15Settings_DialogE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject Views::Settings_Dialog::staticMetaObject = { {
    QMetaObject::SuperData::link<Views::Multi_Page_Dialog::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN5Views15Settings_DialogE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN5Views15Settings_DialogE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN5Views15Settings_DialogE_t>.metaTypes,
    nullptr
} };

void Views::Settings_Dialog::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<Settings_Dialog *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->sig_change_mixer_view(); break;
        case 1: _t->sig_change_input(); break;
        case 2: _t->sig_change_tray_visibility(); break;
        case 3: _t->sig_change_tray_icon(); break;
        case 4: _t->sig_change_tray_mixer_dev(); break;
        case 5: _t->sig_change_tray_balloon(); break;
        case 6: _t->sig_change_tray_input(); break;
        case 7: _t->sig_close(); break;
        case 8: _t->wheel_turns_slider_changed((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 9: _t->wheel_turns_input_changed((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 10: _t->tray_wheel_turns_slider_changed((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 11: _t->tray_wheel_turns_input_changed((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 12: _t->change_startup(); break;
        case 13: _t->change_appearance(); break;
        case 14: _t->change_input(); break;
        case 15: _t->change_tray_visibility(); break;
        case 16: _t->change_tray_icon(); break;
        case 17: _t->change_tray_mixer_dev(); break;
        case 18: _t->change_tray_balloon(); break;
        case 19: _t->change_tray_input(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (Settings_Dialog::*)()>(_a, &Settings_Dialog::sig_change_mixer_view, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (Settings_Dialog::*)()>(_a, &Settings_Dialog::sig_change_input, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (Settings_Dialog::*)()>(_a, &Settings_Dialog::sig_change_tray_visibility, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (Settings_Dialog::*)()>(_a, &Settings_Dialog::sig_change_tray_icon, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (Settings_Dialog::*)()>(_a, &Settings_Dialog::sig_change_tray_mixer_dev, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (Settings_Dialog::*)()>(_a, &Settings_Dialog::sig_change_tray_balloon, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (Settings_Dialog::*)()>(_a, &Settings_Dialog::sig_change_tray_input, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (Settings_Dialog::*)()>(_a, &Settings_Dialog::sig_close, 7))
            return;
    }
}

const QMetaObject *Views::Settings_Dialog::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Views::Settings_Dialog::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN5Views15Settings_DialogE_t>.strings))
        return static_cast<void*>(this);
    return Views::Multi_Page_Dialog::qt_metacast(_clname);
}

int Views::Settings_Dialog::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = Views::Multi_Page_Dialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 20)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 20;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 20)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 20;
    }
    return _id;
}

// SIGNAL 0
void Views::Settings_Dialog::sig_change_mixer_view()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void Views::Settings_Dialog::sig_change_input()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void Views::Settings_Dialog::sig_change_tray_visibility()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void Views::Settings_Dialog::sig_change_tray_icon()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void Views::Settings_Dialog::sig_change_tray_mixer_dev()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void Views::Settings_Dialog::sig_change_tray_balloon()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void Views::Settings_Dialog::sig_change_tray_input()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void Views::Settings_Dialog::sig_close()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}
QT_WARNING_POP
