/// QasTools: Desktop toolset for the Linux sound system ALSA.
/// \copyright See COPYING file.

#pragma once

#include "qsnd/ctl_address.hpp"
#include "utility/setup.hpp"

namespace MWdg
{

/// @brief Mixer_Device_Setup
///
class Mixer_Device_Setup : public Utility::Setup
{
  public:
  // -- Construction

  Mixer_Device_Setup ( Utility::Setup * parent_n );

  ~Mixer_Device_Setup ();

  // -- Control address

  const QSnd::Ctl_Address &
  control_address () const
  {
    return _control_address;
  }

  bool
  set_control_address ( const QString & address_n );

  bool
  set_control_address ( const QSnd::Ctl_Address & address_n );

  private:
  // -- Attributes
  QSnd::Ctl_Address _control_address;
};

} // namespace MWdg
