/// QasTools: Desktop toolset for the Linux sound system ALSA.
/// \copyright See COPYING file.

#include "mixer_device_setup.hpp"

namespace MWdg
{

Mixer_Device_Setup::Mixer_Device_Setup ( Utility::Setup * parent_n )
: Utility::Setup ( parent_n )
{
}

Mixer_Device_Setup::~Mixer_Device_Setup () = default;

bool
Mixer_Device_Setup::set_control_address ( const QString & address_n )
{
  if ( _control_address.addr_str () != address_n ) {
    _control_address = address_n;
    notify_change ();
    return true;
  }
  return false;
}

bool
Mixer_Device_Setup::set_control_address ( const QSnd::Ctl_Address & address_n )
{
  return change ( _control_address, address_n );
}

} // namespace MWdg
