/// QasTools: Desktop toolset for the Linux sound system ALSA.
/// \copyright See COPYING file.

#pragma once

#include "mwdg/inputs_setup.hpp"
#include "mwdg/mixer_device_setup.hpp"
#include "utility/setup.hpp"
#include "views/device_selection_view_setup.hpp"
#include "views/mixer_hctl_setup.hpp"
#include <QByteArray>
#include <QKeySequence>

/// @brief Main_Window_Setup
///
class Main_Window_Setup : public Utility::Setup
{
  public:
  // -- Construction

  Main_Window_Setup ();

  ~Main_Window_Setup () override;

  // -- Attributes

  MWdg::Mixer_Device_Setup mixer_dev;
  MWdg::Inputs_Setup inputs;

  Views::Mixer_HCTL_Setup hctl;
  Views::Device_Selection_View_Setup dev_select;

  // -- Show device selection

  bool
  show_dev_select () const
  {
    return _show_dev_select;
  }

  bool
  set_show_dev_select ( bool flag_n );

  private:
  // -- Attributes
  bool _show_dev_select = true;
};
