/****************************************************************************
** Meta object code from reading C++ file 'icon.hpp'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../qastools-v1.5.0/qasmixer/src/tray/icon.hpp"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'icon.hpp' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN4Tray4IconE_t {};
} // unnamed namespace

template <> constexpr inline auto Tray::Icon::qt_create_metaobjectdata<qt_meta_tag_ZN4Tray4IconE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Tray::Icon",
        "sig_hover",
        "",
        "sig_activated",
        "sig_middle_click",
        "sig_wheel_delta",
        "delta_n",
        "sig_quit",
        "activation",
        "QSystemTrayIcon::ActivationReason",
        "reason_n"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'sig_hover'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'sig_activated'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'sig_middle_click'
        QtMocHelpers::SignalData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'sig_wheel_delta'
        QtMocHelpers::SignalData<void(int)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 6 },
        }}),
        // Signal 'sig_quit'
        QtMocHelpers::SignalData<void()>(7, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'activation'
        QtMocHelpers::SlotData<void(QSystemTrayIcon::ActivationReason)>(8, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 9, 10 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<Icon, qt_meta_tag_ZN4Tray4IconE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject Tray::Icon::staticMetaObject = { {
    QMetaObject::SuperData::link<QSystemTrayIcon::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN4Tray4IconE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN4Tray4IconE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN4Tray4IconE_t>.metaTypes,
    nullptr
} };

void Tray::Icon::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<Icon *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->sig_hover(); break;
        case 1: _t->sig_activated(); break;
        case 2: _t->sig_middle_click(); break;
        case 3: _t->sig_wheel_delta((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 4: _t->sig_quit(); break;
        case 5: _t->activation((*reinterpret_cast<std::add_pointer_t<QSystemTrayIcon::ActivationReason>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (Icon::*)()>(_a, &Icon::sig_hover, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (Icon::*)()>(_a, &Icon::sig_activated, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (Icon::*)()>(_a, &Icon::sig_middle_click, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (Icon::*)(int )>(_a, &Icon::sig_wheel_delta, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (Icon::*)()>(_a, &Icon::sig_quit, 4))
            return;
    }
}

const QMetaObject *Tray::Icon::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Tray::Icon::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN4Tray4IconE_t>.strings))
        return static_cast<void*>(this);
    return QSystemTrayIcon::qt_metacast(_clname);
}

int Tray::Icon::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QSystemTrayIcon::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 6)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 6)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 6;
    }
    return _id;
}

// SIGNAL 0
void Tray::Icon::sig_hover()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void Tray::Icon::sig_activated()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void Tray::Icon::sig_middle_click()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void Tray::Icon::sig_wheel_delta(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void Tray::Icon::sig_quit()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}
QT_WARNING_POP
