/// QasTools: Desktop toolset for the Linux sound system ALSA.
/// \copyright See COPYING file.

#include "main_window_setup.hpp"

Main_Window_Setup::Main_Window_Setup ( Utility::Setup * parent_n )
: Utility::Setup ( parent_n )
, mixer_dev ( this )
, inputs ( this )
, mixer ( this )
, dev_select ( this )
, device_settings_dialog ( this )
{
}

Main_Window_Setup::~Main_Window_Setup () = default;

bool
Main_Window_Setup::set_show_dev_select ( bool flag_n )
{
  return change ( _show_dev_select, flag_n );
}

bool
Main_Window_Setup::set_window_state ( const QByteArray & state_n )
{
  return change ( _window_state, state_n );
}

bool
Main_Window_Setup::set_window_geometry ( const QByteArray & geom_n )
{
  return change ( _window_geometry, geom_n );
}

bool
Main_Window_Setup::set_splitter_state ( const QByteArray & state_n )
{
  return change ( _splitter_state, state_n );
}
