/// QasTools: Desktop toolset for the Linux sound system ALSA.
/// \copyright See COPYING file.

#pragma once

#include "utility/setup.hpp"
#include <QKeySequence>
#include <QString>

namespace MWdg
{

/// @brief Inputs_Setup
///
class Inputs_Setup : public Utility::Setup
{
  public:
  // -- Construction

  Inputs_Setup ( Utility::Setup * parent_n );

  ~Inputs_Setup ();

  // -- Wheel degrees

  unsigned int
  wheel_degrees () const
  {
    return _wheel_degrees;
  }

  bool
  set_wheel_degrees ( unsigned int degrees_n );

  // -- Translation

  void
  update_translation ();

  // -- Attributes

  QKeySequence ks_toggle_joined;
  QKeySequence ks_level_channels;
  QKeySequence ks_mute_volumes;

  QKeySequence ks_toggle_vis_stream[ 2 ];

  QString ts_split_channels;
  QString ts_join_channels;
  QString ts_level_channels;

  private:
  // -- Attributes
  unsigned int _wheel_degrees = 720;
};

} // namespace MWdg
