/// QasTools: Desktop toolset for the Linux sound system ALSA.
/// \copyright See COPYING file.

#pragma once

#include <QDBusAbstractAdaptor>

// -- Forward declaration
class Desktop_Items;

/// @brief DBus Adaptor for the desktop items
///
class Desktop_Items_DBus_Adaptor : public QDBusAbstractAdaptor
{
  Q_OBJECT
  Q_CLASSINFO ( "D-Bus Interface", "org.QasTools.QasMixer" )

  public:
  // -- Construction

  Desktop_Items_DBus_Adaptor ( Desktop_Items * desktop_items_n );

  ~Desktop_Items_DBus_Adaptor () override;

  // -- Slots

  Q_SLOT
  void
  raise ();

  Q_SLOT
  void
  selectControl ( const QString & ctl_name_n );

  public:
  // -- Attributes
  Desktop_Items * _desktop_items = nullptr;
};
